#ifndef MPC_AVOIDOBS_STRUCT_H
#define MPC_AVOIDOBS_STRUCT_H

/* #include "MPC_controller-Struct.h" */
#include <MPC/mpc-gen.hpp>
#include "genPosStruct.h"

/***********************************************************************/
/* MPC_avoidObs running mode */

#define AVOIDOBS_NOT_RUNNING 0
#define AVOIDOBS_TRAJECTORY_SELECTION 1
#define AVOIDOBS_WP_POSTER_TRACKING 2
#define AVOIDOBS_SIMPLE_GOTO 3
#define AVOIDOBS_SIMPLE_FOLLOW_TRAJ 4
#define AVOIDOBS_AVOID_TRAJGEN 5

#define MPC_AVOIDOBS_MODE int


/***********************************************************************/
/* Robotic system type */

#define MPC_OA_UNDEFINED_SYS 0
#define MPC_OA_MANA_SYS 1

#define OA_ROBOTIC_SYS_T int


/***********************************************************************/
/* Velocity feedback mode */
#define VEL_FB_SENTCTRL 0
#define VEL_FB_GPS 1

#define OA_VEL_FB_T int

/***********************************************************************/
/* Horizon mode */
#define TIME_HORIZON_MODE 1
#define DIST_HORIZON_MODE 2

#define HORIZON_MODE int

/***********************************************************************/
/* Piecewise polynomial trajectory structures */

/* typedef struct POLYNOME3 { */
/*   float a; */
/*   float b; */
/*   float c; */
/* } POLYNOME3; */

typedef struct POLYNOME4 {
  float a;
  float b;
  float c;
  float d;
} POLYNOME4;

typedef struct ITV {
  float binf;
  float bsup;
} ITV;

typedef struct POLY_ITV {
  POLYNOME4 poly;
  ITV itv;
} POLY_ITV;

#define MAX_NB_ITV 15

typedef struct POLY_MULTIITV {
  POLY_ITV tabItv[MAX_NB_ITV];
  int n;
} POLY_MULTIITV;

typedef struct TIMEV {
  long long int sec;
  long long int usec;
} TIMEV;

typedef struct MANA_PCTRAJ
{
  POLY_MULTIITV vel;
  POLY_MULTIITV avel;
/*   double t0; */
/*   double tf; */
  TIMEV t0;
  TIMEV tf;
} MANA_PCTRAJ;

typedef struct MANA_PSTRAJ
{
  GENPOS_CART_CONFIG p0;
  MANA_PCTRAJ pctraj;
} MANA_PSTRAJ;


/***********************************************************************/
/* 2D path tab max size */
#define TAB_CAND_PATH_MAXSIZE 500
#define CAND_PATH_MAXSIZE 15

/* Struct containing a 2D path */
typedef struct CAND_PATH_XY
{
  GENPOS_CART_CONFIG tab[CAND_PATH_MAXSIZE];
  int nbPos;
} CAND_PATH_XY; 

/* Struct containing the candidate paths to follow */
typedef struct TAB_CAND_PATH {
  CAND_PATH_XY tab[TAB_CAND_PATH_MAXSIZE];
  int nbPath;
} TAB_CAND_PATH;


/***********************************************************************/
/* Waypoint structures */

typedef struct WP2D
{
  float x;
  float y;
} WP2D;

#define MAX_NB_WP 50

typedef struct LIST_WP
{
  WP2D tab[MAX_NB_WP];
  int nbWP;
} LIST_WP;


/***********************************************************************/
/**************************** Requests I/O *****************************/
/***********************************************************************/

/***********************************************************************/
/* Initialization parameters */

typedef struct OA_INIT_I
{
/*   char robot[256]; */
  char pomPoster[256];
  char demPoster[256];
  char ctrlPoster[256];
  char distmapPoster[256];
  char gpsPoster[256];
  char strajPoster[256];
  float AO_MAX_DIST_TGT_REACHED;
  int stop_before_col;
} OA_INIT_I;


/***********************************************************************/
/* SetGoToOptions parameters */

typedef struct OA_GOTO_INIT_I
{
  float maxDistTgtReached;
  float maxDistWpReached;
  float maxAngleWpReached;
  int fixFinalTH;
} OA_GOTO_INIT_I;

/***********************************************************************/
/* GoTo / TrajSelTo parameters */

typedef struct TRAJSEL_I
{
  GENPOS_CART_CONFIG goal;
} TRAJSEL_I;


/***********************************************************************/
/* SetManaSys parameters */

typedef struct OA_SETMANASYS_I
{
  double body_length;
  double body_width;
  double body_z;
  double ground_clearance;
  double rollMax;
  double pitchMax;
  double vmin;
  double vmax;
  double accel;
  double decel;
  double wmax;
} OA_SETMANASYS_I;


/***********************************************************************/
/* SetTrajSelOptions parameters */

typedef struct OA_TRAJSEL_OPTIONS_I
{
/*   double AO_WG; */
/*   double AO_WO; */
/*   double AO_WH; */

  double AO_MAX_DIST_TGT_REACHED;
  double AO_MAX_DIST_WP_REACHED;
  double AO_MAX_ANGLE_WP_REACHED;

  double AO_SLOPEMAP_DIST;
  double AO_MAX_SLOPE;
  double AO_MAX_VAR_SLOPE;
  double AO_SAFE_DIST;

  double AO_TIME_TRAJ;
  double AO_TIME_TRAJ_OBS_FREE;

  double AO_DIST_OBS_MAX;
  int AO_SAMPLING_CAND_TRAJ;

/*   int AO_UNKWN_IS_OBS; */

} OA_TRAJSEL_OPTIONS_I;


/***********************************************************************/
/* SetManaTrajSelOptions parameters */

typedef struct OA_MANATRAJSEL_OPTIONS_I
{
  float AO_WG;
  float AO_WO;
  float AO_WH;

  float COEF_ACC;
  float COEF_AACC;
  float COEF_JERK;
  float COEF_AJERK;

  int ManaTrajSel_spg_acc;
  int ManaTrajSel_spg_aacc;
  int ManaTrajSel_spg_jerk;
  int ManaTrajSel_spg_ajerk;
  
} OA_MANATRAJSEL_OPTIONS_I;


/***********************************************************************/
/* Track parameters */

typedef struct TRACK_I
{
  char wpPoster[256];
} TRACK_I;


/***********************************************************************/
/* SetATFParams parameters */

typedef struct ATFPARAMS_I
{
  int version;
  float ds_maxdist_points;
  int min_nbitv_trajgen;
  float coef_time;
} ATFPARAMS_I;

/***********************************************************************/
/* AvoidTF parameters */

typedef struct AVOIDTF_I
{
  char pathPoster[256];
  float width;
  float max_path_length;
/*   float ds_maxdist_points; */
/*   int min_nbitv_trajgen; */
/*   float coef_time; */
  int loop_on_traj;
} AVOIDTF_I;

/***********************************************************************/
/* SetATGParams parameters */

typedef struct ATGPARAMS_I
{
  int port;
  float dt_latShift;
} ROBUFAST_COM_PARAMS_I;

/***********************************************************************/
/* KeepShift (AvoidTF2) parameters */

typedef struct KEEP_SHIFT_I
{
  float shift;
} KEEP_SHIFT_I;

/***********************************************************************/
/* SetTFParams parameters */

typedef struct TFPARAMS_I
{
  float minDistToObstacles;
  float maxDistToObstacles;
  float minCoefSpeed;
} TFPARAMS_I;

/***********************************************************************/
/* FollowTraj parameters */

typedef struct FOLLOWTRAJ_I
{
  HORIZON_MODE horizon_mode;
  float horizon;
  int stop_before_col;
  int loop_on_traj;
} FOLLOWTRAJ_I;

/***********************************************************************/
/* SetDistMapOptions parameters */

typedef struct DISTMAPOPTIONS_I
{
  float SlopeMapDist;
  float MaxSlope;
  float MaxVarSlope;
  float SafeDist;
} DISTMAPOPTIONS_I;

/***********************************************************************/
/* SetVelFeedbackMode parameters */

typedef struct VELFEEDBACKMODE_I {
    char name[256]; 
} VELFEEDBACKMODE_I;

#endif /* MPC_AVOIDOBS_STRUCT_H */
